/**************************************************************************/
/* File: ioctl_remote.c - remote CAN messages support IOCTL systemcall    */
/*                                                                        */
/* LinCAN - (Not only) Linux CAN bus driver                               */
/* Copyright (C) 2002-2009 DCE FEE CTU Prague <http://dce.felk.cvut.cz>   */
/* Copyright (C) 2002-2009 Pavel Pisa <pisa@cmp.felk.cvut.cz>             */
/* Funded by OCERA and FRESCOR IST projects                               */
/* Based on CAN driver code by Arnaud Westenberg <arnaud@wanadoo.nl>      */
/*                                                                        */
/* LinCAN is free software; you can redistribute it and/or modify it      */
/* under terms of the GNU General Public License as published by the      */
/* Free Software Foundation; either version 2, or (at your option) any    */
/* later version.  LinCAN is distributed in the hope that it will be      */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty    */
/* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    */
/* General Public License for more details. You should have received a    */
/* copy of the GNU General Public License along with LinCAN; see file     */
/* COPYING. If not, write to the Free Software Foundation, 675 Mass Ave,  */
/* Cambridge, MA 02139, USA.                                              */
/*                                                                        */
/* To allow use of LinCAN in the compact embedded systems firmware        */
/* and RT-executives (RTEMS for example), main authors agree with next    */
/* special exception:                                                     */
/*                                                                        */
/* Including LinCAN header files in a file, instantiating LinCAN generics */
/* or templates, or linking other files with LinCAN objects to produce    */
/* an application image/executable, does not by itself cause the          */
/* resulting application image/executable to be covered by                */
/* the GNU General Public License.                                        */
/* This exception does not however invalidate any other reasons           */
/* why the executable file might be covered by the GNU Public License.    */
/* Publication of enhanced or derived LinCAN files is required although.  */
/**************************************************************************/

#include "../include/can.h"
#include "../include/can_sysdep.h"
#include "../include/main.h"
#include "../include/read.h"
#include "../include/ioctl.h"

/* This is the 'RTR' read handler for remote transmission request messages */
int can_ioctl_remote_read(struct canuser_t *canuser, struct canmsg_t *rtr_msg,
                          unsigned long rtr_id, int options)
{
	can_spin_irqflags_t flags;
	struct rtr_id *rtr_current, *new_rtr_entry;
	struct msgobj_t *obj;
	struct canchip_t *chip;
	/*struct canque_ends_t *qends;*/
	
	DEBUGMSG("Remote transmission request\n");

	/*qends = canuser->qends;*/

	/* Initialize hardware pointers */
	obj = canuser->msgobj;
	if (obj == NULL) {
		CANMSG("Could not assign buffer structure\n");
		return -ENODEV;
	}

	if ( (chip = obj->hostchip) == NULL) {
		CANMSG("Device is not correctly configured,\n");
		CANMSG("please reload the driver.\n");
		return -ENODEV;
	}

	can_spin_lock_irqsave(&hardware_p->rtr_lock, flags);
	if (hardware_p->rtr_queue == NULL) { //No remote messages pending
		new_rtr_entry=(struct rtr_id *)kmalloc(sizeof(struct rtr_id),GFP_ATOMIC);
		if (new_rtr_entry == NULL) {
			can_spin_unlock_irqrestore(&hardware_p->rtr_lock, 
								flags);
			return -ENOMEM;
		}
		hardware_p->rtr_queue=new_rtr_entry;
	}
	else {
		rtr_current=hardware_p->rtr_queue;
		while (rtr_current->next != NULL)
			rtr_current=rtr_current->next;
		new_rtr_entry=(struct rtr_id *)kmalloc(sizeof(struct rtr_id),GFP_ATOMIC);
		rtr_current->next=new_rtr_entry;
	}
	init_waitqueue_head(&new_rtr_entry->rtr_wq);
	new_rtr_entry->id = rtr_id;
	new_rtr_entry->rtr_message = rtr_msg;
	new_rtr_entry->next=NULL;

	can_spin_unlock_irqrestore(&hardware_p->rtr_lock, flags);

	/* Send remote transmission request */
	chip->chipspecops->remote_request(chip,obj);
	obj->ret = 0;
	interruptible_sleep_on(&new_rtr_entry->rtr_wq);

	can_spin_lock_irqsave(&hardware_p->rtr_lock, flags);
	if (hardware_p->rtr_queue == new_rtr_entry) {
		if (new_rtr_entry->next != NULL) 
			hardware_p->rtr_queue=new_rtr_entry->next;
		else
			hardware_p->rtr_queue=NULL;
	}
	else {
		rtr_current=hardware_p->rtr_queue;
		while (rtr_current->next != new_rtr_entry)
			rtr_current=rtr_current->next;
		if (new_rtr_entry->next != NULL)
			rtr_current->next=new_rtr_entry->next;
		else
			rtr_current->next=NULL;
	}
	can_spin_unlock_irqrestore(&hardware_p->rtr_lock, flags);
	kfree(new_rtr_entry);

	return obj->ret;
}

